﻿
using System;
using System.Drawing;
using System.Windows.Forms;

/*  Approximation of a differential equation by Runge Kutta */
/*                    www.mosismath.com                     */

namespace WindowsFormsApplication1
{
    public delegate void function();

    public partial class MainWin : Form
    {
        public const int datapoints = 50; // number of datapoints for the intergration
        double h = 1.0 / (datapoints - 1); // calculation from 0 to 1.0

        double[] b = new double[datapoints];
        double[] yp = new double[datapoints];
        double[] xp = new double[datapoints];

        public function func;

        double f_xy(double x, double y)
        {
            return -2*x * y * y;
        }
        

        double Interpolate(double xp, int order)
        {
            int i, k;
            double tempYp = 0;
            double yp = 0;
            for (i = 1; i < order; i++)
            {
                tempYp = b[i];
                for (k = 0; k < i; k++)
                {
                    tempYp = tempYp * (xp);
                }
                yp = yp + tempYp;
            }
            return b[0] + yp;
        }
        

        public MainWin()
        {     
            InitializeComponent();
            func = CalcHeun;
        }


        public void DrawGraph(int maxPoints, double minX, double maxX, bool doClear)
        {
            Point p1, p2;
            int j;
            double maxTime = -1.0;
            double maxVal = -1.0;
            double minVal = 1.0;
            double scalefactor;
            bool drawCeroline;
            for (j = 0; j < maxPoints - 1; j++)
            {
                if ((xp[j] >= minX) && (xp[j] <= maxX))
                {
                    if (maxTime < xp[j])
                        maxTime = xp[j];
                    if (maxVal < yp[j])
                        maxVal = yp[j];
                    if (minVal > yp[j])
                        minVal = yp[j];
                }
            }
            maxTime = maxTime * 1.05;
            if (minVal < 0)
            {
                if (maxVal > 0)
                {
                    if (maxVal > Math.Abs(minVal))
                        scalefactor = pGraph.Height / maxVal / 2.2;
                    else
                        scalefactor = pGraph.Height / Math.Abs(minVal) / 2.2;
                }
                else
                    scalefactor = 5.0;
                drawCeroline = true;
            }
            else
            {
                scalefactor = pGraph.Height / (maxVal) / 1.1;
                drawCeroline = false;
            }
            p1 = new Point();
            p2 = new Point();
            Graphics g = pGraph.CreateGraphics();
            if (doClear)
                g.Clear(Color.White);
            Pen bluePen = new Pen(Color.Blue, 2);
            Pen redPen = new Pen(Color.Red, 2);
            Pen blackPen = new Pen(Color.Black, 2);
            if (drawCeroline)
            {
                p1.X = 0;
                p1.Y = (pGraph.Height / 2);
                p2.X = pGraph.Width;
                p2.Y = (pGraph.Height / 2);
                g.DrawLine(blackPen, p1, p2);
            }
            bluePen.Width = 1;
            if (maxTime > 0) // draw interpolated graph
            {
                for (j = 0; j < maxPoints - 1; j++)
                {
                    if ((xp[j] >= minX) && (xp[j] <= maxX))
                    {
                        p1.X = Convert.ToInt32(xp[j] * pGraph.Width / (double)maxTime);
                        p2.X = Convert.ToInt32(xp[j + 1] * pGraph.Width / (double)maxTime);
                        if (drawCeroline)
                        {
                            p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        else
                        {
                            p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        g.DrawLine(bluePen, p1, p2);
                    }
                }
            }
            g.Dispose();
        }


        private void CalcHeun()
        {
            int i;
            double k1;

            xp[0] = 0;
            yp[0] = 1;  // initial value for y[0]
            for (i = 1; i < datapoints; i++)
            {
                xp[i] = i * h;
                k1 = f_xy(xp[i - 1], yp[i - 1]);
                yp[i] = yp[i - 1] + h/2 * (k1 + f_xy(xp[i], yp[i - 1] + h * k1));
            }
        }


        private void CalcRungeKutta2()
        {
            int i;
            double k1, k2;

            xp[0] = 0;
            yp[0] = 1;  // initial value for y[0]
            for (i = 1; i < datapoints; i++)
            {
                xp[i] = i * h;
                k1 = f_xy(xp[i - 1], yp[i - 1]);
                k2 = f_xy(xp[i - 1] + h / 2, yp[i - 1] +  h / 2 * k1);
                yp[i] = yp[i - 1] + h * k2;
            }
        }

        
        private void CalcRungeKutta4()
        {
            int i;
            double k1, k2, k3, k4;
        
            xp[0] = 0;
            yp[0] = 1; // initial value for y[0]
            for (i = 1; i < datapoints; i++)
            {
                xp[i] = i * h;
                k1 = f_xy(xp[i - 1], yp[i - 1]);
                k2 = f_xy(xp[i - 1] + h / 2, yp[i - 1] + h * k1 / 2);
                k3 = f_xy(xp[i - 1] + h / 2, yp[i - 1] + h * (k1 + k2) / 4);
                k4 = f_xy(xp[i - 1] + h, yp[i - 1] + h * (- k2 + 2 * k3));
                yp[i] = yp[i - 1] + h / 6 * (k1 + 4 * k3 + k4);
            }
        }


        private void button1_MouseClick(object sender, MouseEventArgs e)
        {
            func?.Invoke(); //Call delegate
            DrawGraph(datapoints, 0, 1, true);
            tbX.Text = Math.Round(xp[xp.Length - 1], 4).ToString();
            tbY.Text = Math.Round(yp[yp.Length - 1], 8).ToString();
        }


        private void pGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(datapoints, 0, 1, false);
        }


        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (comboBox1.SelectedIndex)
            {
                case 0:
                    func = CalcHeun;
                    break;
                case 1:
                    func = CalcRungeKutta2;
                    break;
                case 2:
                    func = CalcRungeKutta4;
                    break;
            }
        }
    }
}
